@@ -107,6 +107,22 @@ def getLogFile(fileName: str) -> Optional[logging.FileHandler]:
         return None
 
 
+def safeInfo(msg, *args, **kwargs):
+    """Use tqdm redirect to not destroy progress bars"""
+    rootlog = logging.getLogger("")
+    kwargs.setdefault("stacklevel", 2)
+    try:
+        from tqdm.contrib.logging import (  # pylint: disable=import-outside-toplevel
+            logging_redirect_tqdm,
+        )
+    except ModuleNotFoundError:
+        rootlog.info(msg, *args, **kwargs)
+        return
+
+    with logging_redirect_tqdm():
+        rootlog.info(msg, *args, **kwargs)
+
+
 def configLogger(
     args: Namespace, fileLog: Optional[logging.FileHandler], counter: logCount
 ):
