@@ -29,7 +29,9 @@ from libvirtnbdbackup.output.exceptions import OutputException
 from libvirtnbdbackup.ssh.exceptions import sshError
 
 
-def getConfig(args: Namespace, meta: Dict[str, str]) -> List[str]:
+def getConfig(  # pylint: disable=too-many-statements
+    args: Namespace, meta: Dict[str, str]
+) -> List[str]:
     """Check if backup includes exported qcow config and return a list
     of options passed to qemu-img create command"""
     opt: List[str] = []
@@ -78,6 +80,16 @@ def getConfig(args: Namespace, meta: Dict[str, str]) -> List[str]:
             "Unable apply QCOW specific lazy_refcounts option: [%s]", errmsg
         )
 
+    try:
+        cType = qcowConfig["format-specific"]["data"]["compression-type"]
+        opt.append("-o")
+        opt.append(f"compression_type={cType}")
+        logging.info("Setting qcow image compression type: [%s]", cType)
+    except KeyError as errmsg:
+        logging.warning(
+            "Unable apply QCOW specific lazy_refcounts option: [%s]", errmsg
+        )
+
     try:
         dataFile = qcowConfig["format-specific"]["data"]["data-file"]
         if args.adjust_config is True:
