@@ -39,6 +39,11 @@ from libvirtnbdbackup.exceptions import (
 
 log = logging.getLogger()
 
+redefineFlags = (
+    libvirt.VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE
+    | libvirt.VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE_VALIDATE
+)
+
 
 def exists(
     domObj: libvirt.virDomain, checkpointName: str
@@ -59,7 +64,7 @@ def getXml(cptObj: libvirt.virDomainCheckpoint) -> str:
 
 def getSize(domObj: libvirt.virDomain, checkpointName: str) -> int:
     """Return current size of checkpoint for all disks"""
-    size = 0
+    size: int = 0
     cpt = exists(domObj, checkpointName)
     cptTree = xml.asTree(getXml(cpt))
     for s in cptTree.xpath("disks/disk/@size"):
@@ -68,8 +73,10 @@ def getSize(domObj: libvirt.virDomain, checkpointName: str) -> int:
     return size
 
 
-def delete(cptObj: libvirt.virDomainCheckpoint, checkpointName: str) -> bool:
-    """Delete checkpoint"""
+def delete(domObj: libvirt.virDomain, cptObj: libvirt.virDomainCheckpoint) -> bool:
+    """Delete checkpoint or checkpoint metadata in case validation
+    fails (bitmap is missing, but checkpoint still existent)"""
+    flags: int = 0
     checkpointName = cptObj.getName()
     if defaultCheckpointName not in checkpointName:
         log.debug(
@@ -77,9 +84,16 @@ def delete(cptObj: libvirt.virDomainCheckpoint, checkpointName: str) -> bool:
             checkpointName,
         )
         return True
-    log.debug("Attempt to remove checkpoint: [%s]", checkpointName)
+
+    if not validate(domObj, checkpointName):
+        log.warning(
+            "Checkpoint inconsistency detected, removing metadata for checkpoint."
+        )
+        flags = libvirt.VIR_DOMAIN_CHECKPOINT_DELETE_METADATA_ONLY
+
     try:
-        cptObj.delete()
+        log.debug("Attempt to remove checkpoint: [%s]", checkpointName)
+        cptObj.delete(flags)
         log.debug("Removed checkpoint: [%s]", checkpointName)
         return True
     except libvirt.libvirtError as errmsg:
@@ -105,7 +119,7 @@ def backup(args: Namespace, domObj: libvirt.virDomain) -> bool:
         return False
 
 
-def _hasForeign(domObj: libvirt.virDomain, checkpointName: str) -> Optional[str]:
+def _hasForeign(domObj: libvirt.virDomain) -> Optional[str]:
     """Check if the virtual machine has an checkpoint which was not
     created by virtnbdbackup
 
@@ -117,12 +131,14 @@ def _hasForeign(domObj: libvirt.virDomain, checkpointName: str) -> Optional[str]
     match our name, return so.
     """
     cpts = domObj.listAllCheckpoints()
-    if cpts:
-        for cpt in cpts:
-            checkpointName = cpt.getName()
-            log.debug("Found foreign checkpoint: [%s]", checkpointName)
-            if defaultCheckpointName not in checkpointName:
-                return checkpointName
+    if not cpts:
+        return None
+
+    for cpt in cpts:
+        checkpointName = cpt.getName()
+        if defaultCheckpointName not in checkpointName:
+            return checkpointName
+
     return None
 
 
@@ -134,7 +150,7 @@ def checkForeign(
     not originating from this utility"""
     foreign = None
     if args.level in ("full", "inc", "diff"):
-        foreign = _hasForeign(domObj, defaultCheckpointName)
+        foreign = _hasForeign(domObj)
 
     if not foreign:
         return True
@@ -154,13 +170,16 @@ def removeAll(
     domObj: libvirt.virDomain,
     checkpointList: Union[List[Any], None],
     args: Namespace,
-    checkpointName: str,
 ) -> bool:
     """Remove all existing checkpoints for a virtual machine,
-    used during FULL backup to reset checkpoint chain
+    used during FULL backup to reset checkpoint chain, either
+    by using the list from the checkpoint file (for transient
+    checkpoint storage using --checkpointdir) or by using the
+    checkpoints as listed by the libvirt API
     """
-    log.debug("Cleaning up persistent storage %s", args.checkpointdir)
+    log.info("Removing all existent checkpoints before full backup.")
     try:
+        log.debug("Cleaning up persistent storage %s", args.checkpointdir)
         for checkpointFile in glob.glob(f"{args.checkpointdir}/*.xml"):
             log.debug("Remove checkpoint file: %s", checkpointFile)
             os.remove(checkpointFile)
@@ -170,29 +189,32 @@ def removeAll(
 
     if checkpointList is None:
         cpts = domObj.listAllCheckpoints()
-        if cpts:
-            for cpt in cpts:
-                if delete(cpt, checkpointName) is False:
-                    return False
+        if not cpts:
+            return True
+        for cpt in cpts:
+            if delete(domObj, cpt) is False:
+                return False
         return True
 
     for cp in checkpointList:
         cptObj = exists(domObj, cp)
         if cptObj:
-            if delete(cptObj, checkpointName) is False:
+            if delete(domObj, cptObj) is False:
                 return False
     return True
 
 
 def redefine(domObj: libvirt.virDomain, args: Namespace) -> bool:
     """Redefine checkpoints from persistent storage"""
-    log.info("Loading checkpoint list from: [%s]", args.checkpointdir)
     checkpointList = glob.glob(f"{args.checkpointdir}/*.xml")
     checkpointList.sort(key=os.path.getmtime)
 
+    if checkpointList:
+        log.info("Loaded checkpoint list from: [%s]", args.checkpointdir)
+
     for checkpointFile in checkpointList:
-        log.debug("Loading checkpoint config from: [%s]", checkpointFile)
         try:
+            log.debug("Loading checkpoint config from: [%s]", checkpointFile)
             with output.openfile(checkpointFile, "rb") as f:
                 checkpointConfig = f.read()
                 root = ElementTree.fromstring(checkpointConfig)
@@ -225,8 +247,7 @@ def redefine(domObj: libvirt.virDomain, args: Namespace) -> bool:
         try:
             domObj.checkpointCreateXML(
                 checkpointConfig.decode(),
-                libvirt.VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE
-                | libvirt.VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE_VALIDATE,
+                redefineFlags,
             )
         except libvirt.libvirtError as e:
             log.error("Redefining checkpoint failed: [%s]: %s", checkpointName, e)
@@ -267,21 +288,18 @@ def save(args: Namespace) -> None:
 
 
 def validate(domObj: libvirt.virDomain, checkpointName: str) -> bool:
-    """Validate that checkpoint exists"""
+    """Validate that checkpoint and bitmap exist and are OK
+    Currently the only way to correctly verify consistency is by redefine the checkpoint with
+    VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE_VALIDATE option set, which will check the bitmap
+    consistency, too."""
     try:
         c = domObj.checkpointLookupByName(checkpointName)
         checkpointXml = c.getXMLDesc(0)
-        flags = (
-            libvirt.VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE
-            | libvirt.VIR_DOMAIN_CHECKPOINT_CREATE_REDEFINE_VALIDATE
-        )
 
         # Redefine the checkpoint using the provided XML description
-        c = domObj.checkpointCreateXML(checkpointXml, flags)
-        if not c:
+        if not domObj.checkpointCreateXML(checkpointXml, redefineFlags):
             return False
         return True
-
     except libvirt.libvirtError as e:
         log.warning("Failed to validate checkpoint: [%s]", e)
         return False
@@ -301,8 +319,6 @@ def create(
     checkpointName: str = f"{defaultCheckpointName}.0"
     parentCheckpoint: str = ""
     cptFile: str = f"{args.output}/{args.domain}.cpt"
-
-    log.info("Loading checkpoints from: [%s]", cptFile)
     checkpoints: List[str] = read(cptFile)
 
     if args.offline is False:
@@ -312,15 +328,17 @@ def create(
     # save level to reuse it properly when filename is selected for the backup
     args.level_filename = args.level
 
-    log.info("Checkpoint handling.")
+    # remove checkpoints as loaded from the checkpoint json file
     if args.level == "full" and checkpoints:
-        log.info("Removing all existent checkpoints before full backup.")
-        if not removeAll(domObj, checkpoints, args, defaultCheckpointName):
+        log.info("Loaded checkpoints from: [%s]", cptFile)
+        if not removeAll(domObj, checkpoints, args):
             raise RemoveCheckpointError("Failed to remove checkpoint.")
         os.remove(cptFile)
         checkpoints = []
+    # if no checkpoints are found in file, remove all existent ones matching the
+    # name
     elif args.level == "full" and len(checkpoints) < 1:
-        if not removeAll(domObj, None, args, defaultCheckpointName):
+        if not removeAll(domObj, None, args):
             raise RemoveCheckpointError("Failed to remove checkpoint.")
         checkpoints = []
 
@@ -334,14 +352,12 @@ def create(
         if args.offline is True:
             log.info("Offline backup, using latest checkpoint, saving only delta.")
             checkpointName = parentCheckpoint
-
-            # Autostart is disabled, validation could not be performed
+        # Autostart is disabled, validation could not be performed
         elif not validate(domObj, parentCheckpoint):
             log.warning(
                 "Checkpoint [%s] is invalid, switching backup to full.",
                 parentCheckpoint,
             )
-
             args.level = "full"
             # reset back to defaults
             parentCheckpoint = ""
